/*	types.h

	AFAPI types for TTL_PAPERS.

	July 1996 by H. Dietz
*/


#ifndef _P_TYPES_H
#define _P_TYPES_H


/*	Environment variables...
*/
#define	AFAPI		"TTL_PAPERS"	/* The AFAPI mechanism */
#define	AFAPI_VERSION	"961220"	/* The AFAPI version */


/*	Software debounce option...

	Leave P_DEBOUNCE defined if your port hardware otherwise has
	noise-related errors due to God-only-knows-what.  The solution
	used is to make each port input operation keep sampling until
	the same value has been read twice in a row, which should be
	plenty of settling time since this noise generally settles in
	less than 1/10 of the time it takes for a port access.  You
	really should not need P_DEBOUNCE defined.
*/
#define	P_DEBOUNCE
#undef	P_DEBOUNCE


/*	The magic "constants"...
*/
#ifndef	NPROC
#define	NPROC	4			/* Number of PEs */
#endif
extern int iproc;			/* PE number (0..NPROC-1) */
#define	IPROC	((const int) iproc)	/* Safe version of above */
extern int cproc;			/* Console PE number */
#define	CPROC	((const int) cproc)	/* Safe version of above */


/*	Hardware port stuff...
*/
extern unsigned short p_outport;
extern unsigned short p_inport;
extern unsigned short p_modeport;
extern unsigned char p_outlast;
extern unsigned int p_inlast;


/*	AFAPI names for basic data types and values...
*/
typedef	int			p_1u;
typedef	char			p_8;
typedef	unsigned char		p_8u;
typedef	short			p_16;
typedef	unsigned short		p_16u;
typedef	int			p_32;
typedef	unsigned int		p_32u;
typedef	long long		p_64;
typedef	unsigned long long	p_64u;
typedef	float			p_32f;
typedef	double			p_64f;


/*	AFAPI type suffix to unsigned integer suffix conversions...
*/
#define	P_USUFFIX(type)	P_USUFFIX_##type
#define	P_USUFFIX_1u	1u
#define	P_USUFFIX_8	8u
#define	P_USUFFIX_8u	8u
#define	P_USUFFIX_16	16u
#define	P_USUFFIX_16u	16u
#define	P_USUFFIX_32	32u
#define	P_USUFFIX_32u	32u
#define	P_USUFFIX_64	64u
#define	P_USUFFIX_64u	64u


/*	AFAPI types as unsigned integer types...

	The general rule is to stick a "u" on the end of a type name
	to get the unsigned version of the type...  but this rule does
	not work for types that are already unsigned, floats, and
	pointers.  The following macros make it fully consistent.
*/
#define	P_UTYPE(type)	type##u
#define	p_1uu	p_1u
#define	p_8uu	p_8u
#define	p_16uu	p_16u
#define	p_32uu	p_32u
#define	p_64uu	p_64u
#define	p_32fu	p_32u
#define	p_64fu	p_64u


/*	AFAPI type sizes in bits...

	type##_bits gives the number of bits in the type.
*/
#define	p_1u_bits	1
#define	p_8_bits	8
#define	p_8u_bits	8
#define	p_16_bits	16
#define	p_16u_bits	16
#define	p_32_bits	32
#define	p_32u_bits	32
#define	p_64_bits	64
#define	p_64u_bits	64
#define	p_32f_bits	32
#define	p_64f_bits	64


/*	Pointer type...

	Pointers, void *, are special because the number of bits
	needed to represent them can vary.  This is written for 32
	bits, which is usually correct these days.  However, if 16 or
	64 is more appropriate....  It is also ok to have the number
	of bits not be equal to the size of the type; for example, if
	only the low 24 address bits are used, p_ptr_bits can be 24
	even though the unsigned integer data type will be p_32u.

	This defines the type, unsigned int equivalent, and bit size.
*/
typedef	void *		p_ptr;
#define	P_USUFFIX_ptr	32u
#define	p_ptru		p_32u
#define	p_ptr_bits	32


/*	Barrier type...

	This should work up to 64 processors, but GCC seems to be a
	bit funky about handling 64-bit unsigned integers, so for now
	we will limit things to 32 processors.  It would not be
	difficult to convert this into a byte array that would work for
	thousands of processors, but we don't even have 32 to play
	with yet, so we will do that when we have a reason to.

	This defines the type, unsigned int equivalent, and bit size.
	Note that p_bar_bits is ALWAYS equal to NPROC.
*/
#if	NPROC <= 8
typedef	p_8u	p_bar;
#define	P_USUFFIX_bar	8u
#define	p_baru	p_8u
#else
#if	NPROC <= 16
typedef	p_16u	p_bar;
#define	P_USUFFIX_bar	16u
#define	p_baru	p_16u
#else
#if	NPROC <=32
typedef	p_32u	p_bar;
#define	P_USUFFIX_bar	32u
#define	p_baru	p_32u
#else
#error This library currently does not support >32 processors
#endif
#endif
#endif
#define	p_bar_bits	NPROC


#define p_bar_ALL1	((p_bar) (((p_bar) 0) - ((p_bar) 1)))
#define	p_1u_ALL1	1
#define	p_8_ALL1	(-1)
#define	p_8u_ALL1	((p_8u) (((p_8u) 0) - ((p_8u) 1)))
#define	p_16_ALL1	(-1)
#define	p_16u_ALL1	((p_64u) (((p_16u) 0) - ((p_16u) 1)))
#define	p_32_ALL1	(-1)
#define	p_32u_ALL1	((p_64u) (((p_32u) 0) - ((p_32u) 1)))
#define	p_64_ALL1	(-1)
#define	p_64u_ALL1	((p_64u) (((p_64u) 0) - ((p_64u) 1)))
#define p_ptr_ALL1	((p_ptr) (((p_32u) 0) - ((p_32u) 1)))


/*	RSM lock type...

	Really just has to be big enough to hold NPROC+1 distinct
	values...  which means p_8u works up to 255 processors.
*/
typedef	p_8u	s_lock;


/*	The following definitions are used to declare the function
	prototypes in a way that is consistent with the reference card.
	These macros will not work if P_DECLARE is defined now, so we
	also make sure that it isn't defined.
*/
#ifdef	P_DECLARE
#undef	P_DECLARE
#endif

#define	P_DELTA	\
P_DECLARE(1u) \
P_DECLARE(8) \
P_DECLARE(8u) \
P_DECLARE(16) \
P_DECLARE(16u) \
P_DECLARE(32) \
P_DECLARE(32u) \
P_DECLARE(64) \
P_DECLARE(64u)

#define	P_GAMMA \
P_DELTA \
P_DECLARE(32f) \
P_DECLARE(64f)

#define	P_BETA \
P_DELTA \
P_DECLARE(ptr) \
P_DECLARE(bar)

#define	P_ALPHA \
P_DELTA \
P_DECLARE(ptr) \
P_DECLARE(bar) \
P_DECLARE(32f) \
P_DECLARE(64f)


/*	Type casts...

	This is really done primarily so that p_1u need not
	be treated as a special case....
*/
#define	P_CAST_1u(x)	((p_1u) ((x) != 0))
#define	P_CAST_8u(x)	((p_8u) (x))
#define	P_CAST_8(x)	((p_8) (x))
#define	P_CAST_16u(x)	((p_16u) (x))
#define	P_CAST_16(x)	((p_16) (x))
#define	P_CAST_32u(x)	((p_32u) (x))
#define	P_CAST_32(x)	((p_32) (x))
#define	P_CAST_64u(x)	((p_64u) (x))
#define	P_CAST_64(x)	((p_64) (x))
#define	P_CAST_32f(x)	((p_32f) (x))
#define	P_CAST_64f(x)	((p_64f) (x))
#define	P_CAST_ptr(x)	((p_ptr) (x))
#define	P_CAST_bar(x)	((p_bar) (x))


#endif
